==> syncfusion_flutter_maps-18.2.44-beta.tar.gz: lib/src/features/maps_legend.dart <==
part of maps;

class _MapLegendController extends ChangeNotifier {
  List<int> get toggledIndices => _toggledIndices;
  final List<int> _toggledIndices = <int>[];

  void invokeToggleListeners() {
    notifyListeners();
  }

  @override
  void dispose() {
    toggledIndices?.clear();
    super.dispose();
  }
}

/// Visual explanation of the data represented in the map.
///
/// Each item should contains a shape as well as a label.
class _MapLegend extends StatefulWidget {
  const _MapLegend(
    this.mapDataSource,
    this.settings,
    this.shapeDefaultColor,
    this.legendController,
    this.mapsThemeData,
  );

  final dynamic mapDataSource;

  /// Option to customize the legend appearance.
  final MapLegendSettings settings;

  final Color shapeDefaultColor;

  /// Coordinate between map layer and legend.
  final _MapLegendController legendController;

  final SfMapsThemeData mapsThemeData;

  @override
  _MapLegendState createState() => _MapLegendState();
}

class _MapLegendState extends State<_MapLegend> {
  MapLegendSettings settings;

  @override
  Widget build(BuildContext context) {
    settings = widget.settings;
    if (settings.overflowMode == MapLegendOverflowMode.scroll) {
      return SingleChildScrollView(
        scrollDirection: settings.position == MapLegendPosition.top ||
                settings.position == MapLegendPosition.bottom
            ? Axis.horizontal
            : Axis.vertical,
        child: _actualChild(),
      );
    } else {
      return _actualChild();
    }
  }

  /// Configuring the legend items based on the legend settings.
  Widget _actualChild() {
    return Padding(
      padding: settings.padding,
      // Mapped the legend settings properties values to the Wrap widget.
      child: Wrap(
        direction: settings.direction == null
            ? settings.position == MapLegendPosition.top ||
                    settings.position == MapLegendPosition.bottom
                ? Axis.horizontal
                : Axis.vertical
            : settings.direction,
        spacing: settings.itemsSpacing,
        children: _getLegendItems(),
        runSpacing: 6,
        runAlignment: WrapAlignment.center,
        alignment: WrapAlignment.start,
      ),
    );
  }

  /// Returns the list of legend items based on the data source.
  List<Widget> _getLegendItems() {
    final List<Widget> legendItems = <Widget>[];
    final dynamic source = widget.mapDataSource;
    if (source != null && source.isNotEmpty) {
      // Here source is shape color mappers.
      if (source is List) {
        final int length = source.length;
        for (int i = 0; i < length; i++) {
          final MapColorMapper item = source[i];
          assert(item != null);
          final String text =
              item.text ?? item.value ?? '${item.from} - ${item.to}';
          assert(text != null);
          legendItems.add(_legendItem(text, item.color, i));
        }
      } else {
        // Here source is map data source.
        source.forEach((String key, _MapModel mapModel) {
          assert(mapModel.primaryKey != null);
          legendItems.add(_legendItem(
              mapModel.primaryKey, mapModel.shapeColor, mapModel.actualIndex));
        });
      }
    }

    return legendItems;
  }

  /// Returns the legend icon and label.
  Widget _legendItem(String text, Color shapeColor, int index) {
    assert(text != null);

    return _MapLegendItem(
      index: index,
      text: text,
      iconShapeColor: shapeColor ?? widget.shapeDefaultColor,
      legendSettings: settings,
      mapsThemeData: widget.mapsThemeData,
      legendController: widget.legendController,
    );
  }
}

class _MapLegendItem extends LeafRenderObjectWidget {
  const _MapLegendItem({
    this.index,
    this.text,
    this.iconShapeColor,
    this.legendSettings,
    this.mapsThemeData,
    this.legendController,
  });

  final int index;

  final String text;

  final Color iconShapeColor;

  final MapLegendSettings legendSettings;

  final SfMapsThemeData mapsThemeData;

  final _MapLegendController legendController;

  @override
  RenderObject createRenderObject(BuildContext context) {
    return _RenderLegendItem(
      index: index,
      text: text,
      iconShapeColor: iconShapeColor,
      legendSettings: legendSettings,
      mapsThemeData: mapsThemeData,
      legendController: legendController,
      mediaQueryData: MediaQuery.of(context),
    );
  }

  @override
  void updateRenderObject(
      BuildContext context, _RenderLegendItem renderObject) {
    renderObject
      ..text = text
      ..iconShapeColor = iconShapeColor
      ..legendSettings = legendSettings
      ..mapsThemeData = mapsThemeData
      ..mediaQueryData = MediaQuery.of(context);
  }
}

class _RenderLegendItem extends RenderBox {
  _RenderLegendItem({
    int index,
    String text,
    Color iconShapeColor,
    MapLegendSettings legendSettings,
    SfMapsThemeData mapsThemeData,
    _MapLegendController legendController,
    MediaQueryData mediaQueryData,
  })  : _index = index,
        _text = text,
        _iconShapeColor = iconShapeColor,
        _legendSettings = legendSettings,
        _mapsThemeData = mapsThemeData,
        _legendController = legendController,
        _mediaQueryData = mediaQueryData {
    _textPainter = TextPainter(textDirection: TextDirection.ltr);
    _updateTextPainter();
    _tapGestureRecognizer = TapGestureRecognizer()..onTapUp = _onTapUp;
    _showLegendIcon =
        _legendSettings.iconSize != null && _legendSettings.showIcon;
  }

  final int _spacing = 3;

  final int _index;

  final _MapIconShape _iconShape = const _MapIconShape();

  final _MapLegendController _legendController;

  bool _isToggled = false;

  Color _iconColor;

  bool _showLegendIcon;

  TextPainter _textPainter;

  TapGestureRecognizer _tapGestureRecognizer;

  String get text => _text;
  String _text;
  set text(String value) {
    if (_text == value) {
      return;
    }
    _text = value;
    _updateTextPainter();
    markNeedsLayout();
  }

  Color get iconShapeColor => _iconShapeColor;
  Color _iconShapeColor;
  set iconShapeColor(Color value) {
    if (_iconShapeColor == value) {
      return;
    }
    _iconShapeColor = value;
    markNeedsPaint();
  }

  MapLegendSettings get legendSettings => _legendSettings;
  MapLegendSettings _legendSettings;
  set legendSettings(MapLegendSettings value) {
    if (_legendSettings == value) {
      return;
    }
    _legendSettings = value;
    _updateTextPainter();
    _showLegendIcon =
        _legendSettings.iconSize != null && _legendSettings.showIcon;
    markNeedsLayout();
  }

  SfMapsThemeData get mapsThemeData => _mapsThemeData;
  SfMapsThemeData _mapsThemeData;
  set mapsThemeData(SfMapsThemeData value) {
    if (_mapsThemeData == value) {
      return;
    }
    _mapsThemeData = value;
    markNeedsPaint();
  }

  MediaQueryData get mediaQueryData => _mediaQueryData;
  MediaQueryData _mediaQueryData;
  set mediaQueryData(MediaQueryData value) {
    if (_mediaQueryData == value) {
      return;
    }
    _mediaQueryData = value;
    _updateTextPainter();
    markNeedsLayout();
  }

  void _onTapUp(TapUpDetails details) {
    _isToggled = !_legendController._toggledIndices.contains(_index);
    if (_isToggled) {
      _legendController._toggledIndices.add(_index);
    } else {
      _legendController._toggledIndices.remove(_index);
    }
    _legendController.invokeToggleListeners();
    markNeedsPaint();
  }

  void _updateTextPainter() {
    _textPainter.textScaleFactor = _mediaQueryData.textScaleFactor;
    _textPainter.text = TextSpan(text: _text, style: legendSettings.textStyle);
    _textPainter.layout();
  }

  @override
  bool get isRepaintBoundary => true;

  @override
  bool hitTestSelf(Offset position) => _legendSettings.enableToggleInteraction;

  @override
  void handleEvent(PointerEvent event, HitTestEntry entry) {
    if (_legendSettings.enableToggleInteraction &&
        event.down &&
        event is PointerDownEvent) {
      _tapGestureRecognizer.addPointer(event);
    }
  }

  @override
  void performLayout() {
    final double width =
        (_showLegendIcon ? _legendSettings.iconSize.width : 0) +
            _spacing +
            _textPainter.width;
    final double height = max(
            _showLegendIcon ? _legendSettings.iconSize.height : 0,
            _textPainter.height) +
        _spacing;
    size = Size(width, height);
  }

  @override
  void paint(PaintingContext context, Offset offset) {
    double toggledLegendItemOpacity;
    Offset actualOffset;
    if (_isToggled) {
      _iconColor = _legendSettings.toggledShapeColor;
      toggledLegendItemOpacity = 0.5;
    } else {
      _iconColor = _iconShapeColor;
      toggledLegendItemOpacity = 1;
    }
    if (_showLegendIcon) {
      final Size halfIconSize = _iconShape.getPreferredSize(
              _legendSettings.iconSize, _mapsThemeData) /
          2;
      actualOffset = offset +
          Offset(
              halfIconSize.width,
              halfIconSize.height +
                  (size.height - (halfIconSize.height * 2)) / 2);
      _iconShape.paint(context, actualOffset,
          parentBox: this,
          iconSize: _legendSettings.iconSize,
          color: _iconColor,
          iconType: _legendSettings.iconType);
    }

    _textPainter.text = TextSpan(
        style: _legendSettings.textStyle.copyWith(
            color: _legendSettings.textStyle.color
                .withOpacity(toggledLegendItemOpacity)),
        text: _text);
    _textPainter.layout();
    actualOffset = offset +
        Offset(
            (_showLegendIcon ? _legendSettings.iconSize.width : 0.0) + _spacing,
            (size.height - _textPainter.height) / 2);
    _textPainter.paint(context.canvas, actualOffset);
  }
}

==> syncfusion_flutter_maps-18.2.55-beta.tar.gz: lib/src/features/maps_legend.dart <==
part of maps;

class _MapLegendController extends ChangeNotifier {
  List<int> get toggledIndices => _toggledIndices;
  final List<int> _toggledIndices = <int>[];

  void invokeToggleListeners() {
    notifyListeners();
  }

  @override
  void dispose() {
    toggledIndices?.clear();
    super.dispose();
  }
}

/// Visual explanation of the data represented in the map.
///
/// Each item should contains a shape as well as a label.
class _MapLegend extends StatefulWidget {
  const _MapLegend(
    this.mapDataSource,
    this.settings,
    this.shapeDefaultColor,
    this.legendController,
    this.mapsThemeData,
  );

  final dynamic mapDataSource;

  /// Option to customize the legend appearance.
  final MapLegendSettings settings;

  final Color shapeDefaultColor;

  /// Coordinate between map layer and legend.
  final _MapLegendController legendController;

  final SfMapsThemeData mapsThemeData;

  @override
  _MapLegendState createState() => _MapLegendState();
}

class _MapLegendState extends State<_MapLegend> {
  MapLegendSettings settings;

  @override
  Widget build(BuildContext context) {
    settings = widget.settings;
    if (settings.overflowMode == MapLegendOverflowMode.scroll) {
      return SingleChildScrollView(
        scrollDirection: settings.position == MapLegendPosition.top ||
                settings.position == MapLegendPosition.bottom
            ? Axis.horizontal
            : Axis.vertical,
        child: _actualChild(),
      );
    } else {
      return _actualChild();
    }
  }

  /// Configuring the legend items based on the legend settings.
  Widget _actualChild() {
    return Padding(
      padding: settings.padding,
      // Mapped the legend settings properties values to the Wrap widget.
      child: Wrap(
        direction: settings.direction ??
                settings.position == MapLegendPosition.top ||
                    settings.position == MapLegendPosition.bottom
            ? Axis.horizontal
            : Axis.vertical,
        spacing: settings.itemsSpacing,
        children: _getLegendItems(),
        runSpacing: 6,
        runAlignment: WrapAlignment.center,
        alignment: WrapAlignment.start,
      ),
    );
  }

  /// Returns the list of legend items based on the data source.
  List<Widget> _getLegendItems() {
    final List<Widget> legendItems = <Widget>[];
    final dynamic source = widget.mapDataSource;
    if (source != null && source.isNotEmpty) {
      // Here source is shape color mappers.
      if (source is List) {
        final int length = source.length;
        for (int i = 0; i < length; i++) {
          final MapColorMapper item = source[i];
          assert(item != null);
          final String text =
              item.text ?? item.value ?? '${item.from} - ${item.to}';
          assert(text != null);
          legendItems.add(_legendItem(text, item.color, i));
        }
      } else {
        // Here source is map data source.
        source.forEach((String key, _MapModel mapModel) {
          assert(mapModel.primaryKey != null);
          legendItems.add(_legendItem(
              mapModel.primaryKey, mapModel.shapeColor, mapModel.actualIndex));
        });
      }
    }

    return legendItems;
  }

  /// Returns the legend icon and label.
  Widget _legendItem(String text, Color shapeColor, int index) {
    assert(text != null);

    return _MapLegendItem(
      index: index,
      text: text,
      iconShapeColor: shapeColor ?? widget.shapeDefaultColor,
      legendSettings: settings,
      mapsThemeData: widget.mapsThemeData,
      legendController: widget.legendController,
    );
  }
}

class _MapLegendItem extends LeafRenderObjectWidget {
  const _MapLegendItem({
    this.index,
    this.text,
    this.iconShapeColor,
    this.legendSettings,
    this.mapsThemeData,
    this.legendController,
  });

  final int index;

  final String text;

  final Color iconShapeColor;

  final MapLegendSettings legendSettings;

  final SfMapsThemeData mapsThemeData;

  final _MapLegendController legendController;

  @override
  RenderObject createRenderObject(BuildContext context) {
    return _RenderLegendItem(
      index: index,
      text: text,
      iconShapeColor: iconShapeColor,
      legendSettings: legendSettings,
      mapsThemeData: mapsThemeData,
      legendController: legendController,
      mediaQueryData: MediaQuery.of(context),
    );
  }

  @override
  void updateRenderObject(
      BuildContext context, _RenderLegendItem renderObject) {
    renderObject
      ..text = text
      ..iconShapeColor = iconShapeColor
      ..legendSettings = legendSettings
      ..mapsThemeData = mapsThemeData
      ..mediaQueryData = MediaQuery.of(context);
  }
}

class _RenderLegendItem extends RenderBox {
  _RenderLegendItem({
    int index,
    String text,
    Color iconShapeColor,
    MapLegendSettings legendSettings,
    SfMapsThemeData mapsThemeData,
    _MapLegendController legendController,
    MediaQueryData mediaQueryData,
  })  : _index = index,
        _text = text,
        _iconShapeColor = iconShapeColor,
        _legendSettings = legendSettings,
        _mapsThemeData = mapsThemeData,
        _legendController = legendController,
        _mediaQueryData = mediaQueryData {
    _textPainter = TextPainter(textDirection: TextDirection.ltr);
    _updateTextPainter();
    _tapGestureRecognizer = TapGestureRecognizer()..onTapUp = _onTapUp;
    _showLegendIcon =
        _legendSettings.iconSize != null && _legendSettings.showIcon;
  }

  final int _spacing = 3;

  final int _index;

  final _MapIconShape _iconShape = const _MapIconShape();

  final _MapLegendController _legendController;

  bool _isToggled = false;

  Color _iconColor;

  bool _showLegendIcon;

  TextPainter _textPainter;

  TapGestureRecognizer _tapGestureRecognizer;

  String get text => _text;
  String _text;
  set text(String value) {
    if (_text == value) {
      return;
    }
    _text = value;
    _updateTextPainter();
    markNeedsLayout();
  }

  Color get iconShapeColor => _iconShapeColor;
  Color _iconShapeColor;
  set iconShapeColor(Color value) {
    if (_iconShapeColor == value) {
      return;
    }
    _iconShapeColor = value;
    markNeedsPaint();
  }

  MapLegendSettings get legendSettings => _legendSettings;
  MapLegendSettings _legendSettings;
  set legendSettings(MapLegendSettings value) {
    if (_legendSettings == value) {
      return;
    }
    _legendSettings = value;
    _updateTextPainter();
    _showLegendIcon =
        _legendSettings.iconSize != null && _legendSettings.showIcon;
    markNeedsLayout();
  }

  SfMapsThemeData get mapsThemeData => _mapsThemeData;
  SfMapsThemeData _mapsThemeData;
  set mapsThemeData(SfMapsThemeData value) {
    if (_mapsThemeData == value) {
      return;
    }
    _mapsThemeData = value;
    markNeedsPaint();
  }

  MediaQueryData get mediaQueryData => _mediaQueryData;
  MediaQueryData _mediaQueryData;
  set mediaQueryData(MediaQueryData value) {
    if (_mediaQueryData == value) {
      return;
    }
    _mediaQueryData = value;
    _updateTextPainter();
    markNeedsLayout();
  }

  void _onTapUp(TapUpDetails details) {
    _isToggled = !_legendController._toggledIndices.contains(_index);
    if (_isToggled) {
      _legendController._toggledIndices.add(_index);
    } else {
      _legendController._toggledIndices.remove(_index);
    }
    _legendController.invokeToggleListeners();
    markNeedsPaint();
  }

  void _updateTextPainter() {
    _textPainter.textScaleFactor = _mediaQueryData.textScaleFactor;
    _textPainter.text = TextSpan(text: _text, style: legendSettings.textStyle);
    _textPainter.layout();
  }

  @override
  bool get isRepaintBoundary => true;

  @override
  bool hitTestSelf(Offset position) => _legendSettings.enableToggleInteraction;

  @override
  void handleEvent(PointerEvent event, HitTestEntry entry) {
    if (_legendSettings.enableToggleInteraction &&
        event.down &&
        event is PointerDownEvent) {
      _tapGestureRecognizer.addPointer(event);
    }
  }

  @override
  void performLayout() {
    final double width =
        (_showLegendIcon ? _legendSettings.iconSize.width : 0) +
            _spacing +
            _textPainter.width;
    final double height = max(
            _showLegendIcon ? _legendSettings.iconSize.height : 0,
            _textPainter.height) +
        _spacing;
    size = Size(width, height);
  }

  @override
  void paint(PaintingContext context, Offset offset) {
    double toggledLegendItemOpacity;
    Offset actualOffset;
    if (_isToggled) {
      _iconColor = _legendSettings.toggledShapeColor;
      toggledLegendItemOpacity = 0.5;
    } else {
      _iconColor = _iconShapeColor;
      toggledLegendItemOpacity = 1;
    }
    if (_showLegendIcon) {
      final Size halfIconSize = _iconShape.getPreferredSize(
              _legendSettings.iconSize, _mapsThemeData) /
          2;
      actualOffset = offset +
          Offset(
              halfIconSize.width,
              halfIconSize.height +
                  (size.height - (halfIconSize.height * 2)) / 2);
      _iconShape.paint(context, actualOffset,
          parentBox: this,
          iconSize: _legendSettings.iconSize,
          color: _iconColor,
          iconType: _legendSettings.iconType);
    }

    _textPainter.text = TextSpan(
        style: _legendSettings.textStyle.copyWith(
            color: _legendSettings.textStyle.color
                .withOpacity(toggledLegendItemOpacity)),
        text: _text);
    _textPainter.layout();
    actualOffset = offset +
        Offset(
            (_showLegendIcon ? _legendSettings.iconSize.width : 0.0) + _spacing,
            (size.height - _textPainter.height) / 2);
    _textPainter.paint(context.canvas, actualOffset);
  }
}
